@@ -1,20 +1,16 @@
 import keyword
 import logging
 import re
-import warnings
 
 from lxml import etree
 
-from zeep import exceptions
-from zeep.exceptions import XMLParseError, ZeepWarning
-from zeep.parser import absolute_location
+from zeep.exceptions import XMLParseError
+from zeep.loader import absolute_location, load_external
 from zeep.utils import as_qname, qname_attr
-from zeep.xsd import builtins as xsd_builtins
 from zeep.xsd import elements as xsd_elements
-from zeep.xsd import indicators as xsd_indicators
 from zeep.xsd import types as xsd_types
 from zeep.xsd.const import xsd_ns
-from zeep.xsd.parser import load_external
+from zeep.xsd.types.unresolved import UnresolvedCustomType, UnresolvedType
 
 logger = logging.getLogger(__name__)
 
@@ -40,13 +36,36 @@ class SchemaVisitor(object):
     """Visitor which processes XSD files and registers global elements and
     types in the given schema.
 
+    :param schema:
+    :type schema: zeep.xsd.schema.Schema
+    :param document:
+    :type document: zeep.xsd.schema.SchemaDocument
+
     """
-    def __init__(self, document, parser_context=None):
+
+    def __init__(self, schema, document):
         self.document = document
-        self.schema = document._schema
-        self.parser_context = parser_context
+        self.schema = schema
         self._includes = set()
 
+    def register_element(self, qname, instance):
+        self.document.register_element(qname, instance)
+
+    def register_attribute(self, name, instance):
+        self.document.register_attribute(name, instance)
+
+    def register_type(self, qname, instance):
+        self.document.register_type(qname, instance)
+
+    def register_group(self, qname, instance):
+        self.document.register_group(qname, instance)
+
+    def register_attribute_group(self, qname, instance):
+        self.document.register_attribute_group(qname, instance)
+
+    def register_import(self, namespace, document):
+        self.document.register_import(namespace, document)
+
     def process(self, node, parent):
         visit_func = self.visitors.get(node.tag)
         if not visit_func:
@@ -83,7 +102,10 @@ class SchemaVisitor(object):
         return cls(node.tag, ref, self.schema, **kwargs)
 
     def visit_schema(self, node):
-        """
+        """Visit the xsd:schema element and process all the child elements
+
+        Definition::
+
             <schema
               attributeFormDefault = (qualified | unqualified): unqualified
               blockDefault = (#all | List of (extension | restriction | substitution) : ''
@@ -101,6 +123,9 @@ class SchemaVisitor(object):
                  annotation*)*)
             </schema>
 
+        :param node: The XML node
+        :type node: lxml.etree._Element
+
         """
         assert node is not None
 
@@ -108,12 +133,14 @@ class SchemaVisitor(object):
         self.document._element_form = node.get('elementFormDefault', 'unqualified')
         self.document._attribute_form = node.get('attributeFormDefault', 'unqualified')
 
-        parent = node
-        for node in node.iterchildren():
-            self.process(node, parent=parent)
+        for child in node:
+            self.process(child, parent=node)
 
     def visit_import(self, node, parent):
         """
+
+        Definition::
+
             <import
               id = ID
               namespace = anyURI
@@ -121,6 +148,12 @@ class SchemaVisitor(object):
               {any attributes with non-schema Namespace}...>
             Content: (annotation?)
             </import>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         schema_node = None
         namespace = node.get('namespace')
@@ -131,25 +164,17 @@ class SchemaVisitor(object):
         if not namespace and not self.document._target_namespace:
             raise XMLParseError(
                 "The attribute 'namespace' must be existent if the "
-                "importing schema has no target namespace.")
+                "importing schema has no target namespace.",
+                filename=self._document.location,
+                sourceline=node.sourceline)
 
         # Check if the schema is already imported before based on the
         # namespace. Schema's without namespace are registered as 'None'
-        schema = self.parser_context.schema_objects.get(namespace)
-        if schema:
-            if location and schema._location != location:
-                # Use same warning message as libxml2
-                message = (
-                    "Skipping import of schema located at %r " +
-                    "for the namespace %r, since the namespace was " +
-                    "already imported with the schema located at %r"
-                    ) % (location, namespace or '(null)', schema._location)
-                warnings.warn(message, ZeepWarning, stacklevel=6)
-
-                return
+        document = self.schema._get_schema_document(namespace, location)
+        if document:
             logger.debug("Returning existing schema: %r", location)
-            self.document._imports[namespace] = schema
-            return schema
+            self.register_import(namespace, document)
+            return document
 
         # Hardcode the mapping between the xml namespace and the xsd for now.
         # This seems to fix issues with exchange wsdl's, see #220
@@ -166,7 +191,9 @@ class SchemaVisitor(object):
 
         # Load the XML
         schema_node = load_external(
-            location, self.document._transport, self.parser_context)
+            location,
+            self.schema._transport,
+            strict=self.schema.strict)
 
         # Check if the xsd:import namespace matches the targetNamespace. If
         # the xsd:import statement didn't specify a namespace then make sure
@@ -176,38 +203,31 @@ class SchemaVisitor(object):
             raise XMLParseError((
                 "The namespace defined on the xsd:import doesn't match the "
                 "imported targetNamespace located at %r "
-                ) % (location))
-        elif schema_tns in self.parser_context.schema_objects:
-            schema = self.parser_context.schema_objects.get(schema_tns)
-            message = (
-                "Skipping import of schema located at %r " +
-                "for the namespace %r, since the namespace was " +
-                "already imported with the schema located at %r"
-                ) % (location, namespace or '(null)', schema._location)
-            warnings.warn(message, ZeepWarning, stacklevel=6)
+                ) % (location),
+                filename=self.document._location,
+                sourceline=node.sourceline)
 
-        # If this schema location is 'internal' then retrieve the original
-        # location since that is used as base url for sub include/imports
-        if location in self.parser_context.schema_locations:
-            base_url = self.parser_context.schema_locations[location]
-        else:
-            base_url = location
-
-        schema = self.document.__class__(
-            schema_node, self.document._transport, self.schema, location,
-            self.parser_context, base_url)
-
-        self.document._imports[namespace] = schema
+        schema = self.schema.create_new_document(schema_node, location)
+        self.register_import(namespace, schema)
         return schema
 
     def visit_include(self, node, parent):
         """
-        <include
-          id = ID
-          schemaLocation = anyURI
-          {any attributes with non-schema Namespace}...>
-        Content: (annotation?)
-        </include>
+
+        Definition::
+
+            <include
+              id = ID
+              schemaLocation = anyURI
+              {any attributes with non-schema Namespace}...>
+            Content: (annotation?)
+            </include>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         if not node.get('schemaLocation'):
             raise NotImplementedError("schemaLocation is required")
@@ -217,14 +237,49 @@ class SchemaVisitor(object):
             return
 
         schema_node = load_external(
-            location, self.document._transport, self.parser_context,
-            base_url=self.document._base_url)
+            location, self.schema._transport,
+            base_url=self.document._base_url,
+            strict=self.schema.strict)
         self._includes.add(location)
 
-        return self.visit_schema(schema_node)
+        # When the included document has no default namespace defined but the
+        # parent document does have this then we should (atleast for #360)
+        # transfer the default namespace to the included schema. We can't
+        # update the nsmap of elements in lxml so we create a new schema with
+        # the correct nsmap and move all the content there.
+        if not schema_node.nsmap.get(None) and node.nsmap.get(None):
+            nsmap = {None: node.nsmap[None]}
+            nsmap.update(schema_node.nsmap)
+            new = etree.Element(schema_node.tag, nsmap=nsmap)
+            for child in schema_node:
+                new.append(child)
+            for key, value in schema_node.attrib.items():
+                new.set(key, value)
+            schema_node = new
+
+        # Use the element/attribute form defaults from the schema while
+        # processing the nodes.
+        element_form_default = self.document._element_form
+        attribute_form_default = self.document._attribute_form
+        base_url = self.document._base_url
+
+        self.document._element_form = schema_node.get('elementFormDefault', 'unqualified')
+        self.document._attribute_form = schema_node.get('attributeFormDefault', 'unqualified')
+        self.document._base_url = absolute_location(location, self.document._base_url)
+
+        # Iterate directly over the children.
+        for child in schema_node:
+            self.process(child, parent=schema_node)
+
+        self.document._element_form = element_form_default
+        self.document._attribute_form = attribute_form_default
+        self.document._base_url = base_url
 
     def visit_element(self, node, parent):
         """
+
+        Definition::
+
             <element
               abstract = Boolean : false
               block = (#all | List of (extension | restriction | substitution))
@@ -244,6 +299,12 @@ class SchemaVisitor(object):
             Content: (annotation?, (
                       (simpleType | complexType)?, (unique | key | keyref)*))
             </element>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         is_global = parent.tag == tags.schema
 
@@ -288,7 +349,7 @@ class SchemaVisitor(object):
             if node_type:
                 xsd_type = self._get_type(node_type.text)
             else:
-                xsd_type = xsd_builtins.AnyType()
+                xsd_type = xsd_types.AnyType()
 
         # Naive workaround to mark fields which are part of a choice element
         # as optional
@@ -302,16 +363,16 @@ class SchemaVisitor(object):
             min_occurs=min_occurs, max_occurs=max_occurs, nillable=nillable,
             default=default, is_global=is_global)
 
-        self.document._elm_instances.append(element)
-
         # Only register global elements
         if is_global:
-            self.document.register_element(qname, element)
+            self.register_element(qname, element)
         return element
 
     def visit_attribute(self, node, parent):
         """Declares an attribute.
 
+        Definition::
+
             <attribute
               default = string
               fixed = string
@@ -324,6 +385,12 @@ class SchemaVisitor(object):
               {any attributes with non-schema Namespace...}>
             Content: (annotation?, (simpleType?))
             </attribute>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         is_global = parent.tag == tags.schema
 
@@ -333,9 +400,8 @@ class SchemaVisitor(object):
             match = re.match('([^\[]+)', array_type)
             if match:
                 array_type = match.groups()[0]
-                qname = as_qname(
-                    array_type, node.nsmap, self.document._target_namespace)
-                array_type = xsd_types.UnresolvedType(qname, self.schema)
+                qname = as_qname(array_type, node.nsmap)
+                array_type = UnresolvedType(qname, self.schema)
 
         # If the elment has a ref attribute then all other attributes cannot
         # be present. Short circuit that here.
@@ -346,9 +412,8 @@ class SchemaVisitor(object):
                 return result
 
         attribute_form = node.get('form', self.document._attribute_form)
-        qname = qname_attr(node, 'name', self.document._target_namespace)
         if attribute_form == 'qualified' or is_global:
-            name = qname
+            name = qname_attr(node, 'name', self.document._target_namespace)
         else:
             name = etree.QName(node.get('name'))
 
@@ -360,7 +425,7 @@ class SchemaVisitor(object):
             if node_type:
                 xsd_type = self._get_type(node_type)
             else:
-                xsd_type = xsd_builtins.AnyType()
+                xsd_type = xsd_types.AnyType()
 
         # TODO: We ignore 'prohobited' for now
         required = node.get('use') == 'required'
@@ -368,15 +433,16 @@ class SchemaVisitor(object):
 
         attr = xsd_elements.Attribute(
             name, type_=xsd_type, default=default, required=required)
-        self.document._elm_instances.append(attr)
 
         # Only register global elements
         if is_global:
-            self.document.register_attribute(qname, attr)
+            self.register_attribute(name, attr)
         return attr
 
     def visit_simple_type(self, node, parent):
         """
+        Definition::
+
             <simpleType
               final = (#all | (list | union | restriction))
               id = ID
@@ -384,6 +450,12 @@ class SchemaVisitor(object):
               {any attributes with non-schema Namespace}...>
             Content: (annotation?, (restriction | list | union))
             </simpleType>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
 
         if parent.tag == tags.schema:
@@ -399,8 +471,7 @@ class SchemaVisitor(object):
         child = items[0]
         if child.tag == tags.restriction:
             base_type = self.visit_restriction_simple_type(child, node)
-            xsd_type = xsd_types.UnresolvedCustomType(
-                qname, base_type, self.schema)
+            xsd_type = UnresolvedCustomType(qname, base_type, self.schema)
 
         elif child.tag == tags.list:
             xsd_type = self.visit_list(child, node)
@@ -412,23 +483,30 @@ class SchemaVisitor(object):
 
         assert xsd_type is not None
         if is_global:
-            self.document.register_type(qname, xsd_type)
+            self.register_type(qname, xsd_type)
         return xsd_type
 
     def visit_complex_type(self, node, parent):
         """
-        <complexType
-          abstract = Boolean : false
-          block = (#all | List of (extension | restriction))
-          final = (#all | List of (extension | restriction))
-          id = ID
-          mixed = Boolean : false
-          name = NCName
-          {any attributes with non-schema Namespace...}>
-        Content: (annotation?, (simpleContent | complexContent |
-                  ((group | all | choice | sequence)?,
-                  ((attribute | attributeGroup)*, anyAttribute?))))
-        </complexType>
+        Definition::
+
+            <complexType
+              abstract = Boolean : false
+              block = (#all | List of (extension | restriction))
+              final = (#all | List of (extension | restriction))
+              id = ID
+              mixed = Boolean : false
+              name = NCName
+              {any attributes with non-schema Namespace...}>
+            Content: (annotation?, (simpleContent | complexContent |
+                      ((group | all | choice | sequence)?,
+                      ((attribute | attributeGroup)*, anyAttribute?))))
+            </complexType>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
 
         """
         children = []
@@ -478,22 +556,30 @@ class SchemaVisitor(object):
                 element=element, attributes=attributes, qname=qname,
                 is_global=is_global)
         else:
-            xsd_type = xsd_cls(qname=qname)
+            xsd_type = xsd_cls(qname=qname, is_global=is_global)
 
         if is_global:
-            self.document.register_type(qname, xsd_type)
+            self.register_type(qname, xsd_type)
         return xsd_type
 
-    def visit_complex_content(self, node, parent, namespace=None):
+    def visit_complex_content(self, node, parent):
         """The complexContent element defines extensions or restrictions on a
         complex type that contains mixed content or elements only.
 
+        Definition::
+
             <complexContent
               id = ID
               mixed = Boolean
               {any attributes with non-schema Namespace}...>
             Content: (annotation?,  (restriction | extension))
             </complexContent>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
 
         child = node.getchildren()[-1]
@@ -515,16 +601,24 @@ class SchemaVisitor(object):
                 'extension': base,
             }
 
-    def visit_simple_content(self, node, parent, namespace=None):
+    def visit_simple_content(self, node, parent):
         """Contains extensions or restrictions on a complexType element with
         character data or a simpleType element as content and contains no
         elements.
 
+        Definition::
+
             <simpleContent
               id = ID
               {any attributes with non-schema Namespace}...>
             Content: (annotation?, (restriction | extension))
             </simpleContent>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
 
         child = node.getchildren()[-1]
@@ -535,8 +629,10 @@ class SchemaVisitor(object):
             return self.visit_extension_simple_content(child, node)
         raise AssertionError("Expected restriction or extension")
 
-    def visit_restriction_simple_type(self, node, parent, namespace=None):
+    def visit_restriction_simple_type(self, node, parent):
         """
+        Definition::
+
             <restriction
               base = QName
               id = ID
@@ -547,6 +643,12 @@ class SchemaVisitor(object):
                     totalDigits |fractionDigits | length | minLength |
                     maxLength | enumeration | whiteSpace | pattern)*))
             </restriction>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         base_name = qname_attr(node, 'base')
         if base_name:
@@ -556,8 +658,10 @@ class SchemaVisitor(object):
         if children[0].tag == tags.simpleType:
             return self.visit_simple_type(children[0], node)
 
-    def visit_restriction_simple_content(self, node, parent, namespace=None):
+    def visit_restriction_simple_content(self, node, parent):
         """
+        Definition::
+
             <restriction
               base = QName
               id = ID
@@ -569,14 +673,22 @@ class SchemaVisitor(object):
                     maxLength | enumeration | whiteSpace | pattern)*
                 )?, ((attribute | attributeGroup)*, anyAttribute?))
             </restriction>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         base_name = qname_attr(node, 'base')
         base_type = self._get_type(base_name)
         return base_type, []
 
-    def visit_restriction_complex_content(self, node, parent, namespace=None):
+    def visit_restriction_complex_content(self, node, parent):
         """
 
+        Definition::
+
             <restriction
               base = QName
               id = ID
@@ -584,6 +696,12 @@ class SchemaVisitor(object):
             Content: (annotation?, (group | all | choice | sequence)?,
                     ((attribute | attributeGroup)*, anyAttribute?))
             </restriction>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         base_name = qname_attr(node, 'base')
         base_type = self._get_type(base_name)
@@ -602,6 +720,9 @@ class SchemaVisitor(object):
 
     def visit_extension_complex_content(self, node, parent):
         """
+
+        Definition::
+
             <extension
               base = QName
               id = ID
@@ -610,6 +731,12 @@ class SchemaVisitor(object):
                         (group | all | choice | sequence)?,
                         ((attribute | attributeGroup)*, anyAttribute?)))
             </extension>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         base_name = qname_attr(node, 'base')
         base_type = self._get_type(base_name)
@@ -629,6 +756,9 @@ class SchemaVisitor(object):
 
     def visit_extension_simple_content(self, node, parent):
         """
+
+        Definition::
+
             <extension
               base = QName
               id = ID
@@ -646,16 +776,27 @@ class SchemaVisitor(object):
     def visit_annotation(self, node, parent):
         """Defines an annotation.
 
+        Definition::
+
             <annotation
               id = ID
               {any attributes with non-schema Namespace}...>
             Content: (appinfo | documentation)*
             </annotation>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         return
 
     def visit_any(self, node, parent):
         """
+
+        Definition::
+
             <any
               id = ID
               maxOccurs = (nonNegativeInteger | unbounded) : 1
@@ -666,6 +807,12 @@ class SchemaVisitor(object):
               {any attributes with non-schema Namespace...}>
             Content: (annotation?)
             </any>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         min_occurs, max_occurs = _process_occurs_attrs(node)
         process_contents = node.get('processContents', 'strict')
@@ -675,6 +822,8 @@ class SchemaVisitor(object):
 
     def visit_sequence(self, node, parent):
         """
+        Definition::
+
             <sequence
               id = ID
               maxOccurs = (nonNegativeInteger | unbounded) : 1
@@ -683,6 +832,12 @@ class SchemaVisitor(object):
             Content: (annotation?,
                       (element | group | choice | sequence | any)*)
             </sequence>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
 
         sub_types = [
@@ -690,7 +845,7 @@ class SchemaVisitor(object):
             tags.group, tags.sequence
         ]
         min_occurs, max_occurs = _process_occurs_attrs(node)
-        result = xsd_indicators.Sequence(
+        result = xsd_elements.Sequence(
             min_occurs=min_occurs, max_occurs=max_occurs)
 
         annotation, items = self._pop_annotation(node.getchildren())
@@ -707,6 +862,8 @@ class SchemaVisitor(object):
         """Allows the elements in the group to appear (or not appear) in any
         order in the containing element.
 
+        Definition::
+
             <all
               id = ID
               maxOccurs= 1: 1
@@ -714,12 +871,18 @@ class SchemaVisitor(object):
               {any attributes with non-schema Namespace...}>
             Content: (annotation?, element*)
             </all>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
 
         sub_types = [
             tags.annotation, tags.element
         ]
-        result = xsd_indicators.All()
+        result = xsd_elements.All()
 
         for child in node.iterchildren():
             assert child.tag in sub_types, child
@@ -733,6 +896,8 @@ class SchemaVisitor(object):
         """Groups a set of element declarations so that they can be
         incorporated as a group into complex type definitions.
 
+        Definition::
+
             <group
               name= NCName
               id = ID
@@ -744,9 +909,16 @@ class SchemaVisitor(object):
             Content: (annotation?, (all | choice | sequence))
             </group>
 
-        """
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
 
-        result = self.process_reference(node)
+        """
+        min_occurs, max_occurs = _process_occurs_attrs(node)
+
+        result = self.process_reference(
+            node, min_occurs=min_occurs, max_occurs=max_occurs)
         if result:
             return result
 
@@ -757,14 +929,16 @@ class SchemaVisitor(object):
         child = children[0]
 
         item = self.process(child, parent)
-        elm = xsd_indicators.Group(name=qname, child=item)
+        elm = xsd_elements.Group(name=qname, child=item)
 
         if parent.tag == tags.schema:
-            self.document.register_group(qname, elm)
+            self.register_group(qname, elm)
         return elm
 
     def visit_list(self, node, parent):
         """
+        Definition::
+
             <list
               id = ID
               itemType = QName
@@ -775,6 +949,12 @@ class SchemaVisitor(object):
         The use of the simpleType element child and the itemType attribute is
         mutually exclusive.
 
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
+
         """
         item_type = qname_attr(node, 'itemType')
         if item_type:
@@ -787,6 +967,8 @@ class SchemaVisitor(object):
 
     def visit_choice(self, node, parent):
         """
+        Definition::
+
             <choice
               id = ID
               maxOccurs= (nonNegativeInteger | unbounded) : 1
@@ -804,25 +986,33 @@ class SchemaVisitor(object):
         for child in children:
             elm = self.process(child, node)
             choices.append(elm)
-        return xsd_indicators.Choice(
+        return xsd_elements.Choice(
             choices, min_occurs=min_occurs, max_occurs=max_occurs)
 
     def visit_union(self, node, parent):
         """Defines a collection of multiple simpleType definitions.
 
+        Definition::
+
             <union
               id = ID
               memberTypes = List of QNames
               {any attributes with non-schema Namespace}...>
             Content: (annotation?, (simpleType*))
             </union>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         # TODO
         members = node.get('memberTypes')
         types = []
         if members:
             for member in members.split():
-                qname = as_qname(member, node.nsmap, self.document._target_namespace)
+                qname = as_qname(member, node.nsmap)
                 xsd_type = self._get_type(qname)
                 types.append(xsd_type)
         else:
@@ -835,18 +1025,28 @@ class SchemaVisitor(object):
         attribute or element values) must be unique within the specified scope.
         The value must be unique or nil.
 
+        Definition::
+
             <unique
               id = ID
               name = NCName
               {any attributes with non-schema Namespace}...>
             Content: (annotation?, (selector, field+))
             </unique>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         # TODO
         pass
 
     def visit_attribute_group(self, node, parent):
         """
+        Definition::
+
             <attributeGroup
               id = ID
               name = NCName
@@ -855,6 +1055,12 @@ class SchemaVisitor(object):
             Content: (annotation?),
                      ((attribute | attributeGroup)*, anyAttribute?))
             </attributeGroup>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         ref = self.process_reference(node)
         if ref:
@@ -865,10 +1071,12 @@ class SchemaVisitor(object):
 
         attributes = self._process_attributes(node, children)
         attribute_group = xsd_elements.AttributeGroup(qname, attributes)
-        self.document.register_attribute_group(qname, attribute_group)
+        self.register_attribute_group(qname, attribute_group)
 
     def visit_any_attribute(self, node, parent):
         """
+        Definition::
+
             <anyAttribute
               id = ID
               namespace = ((##any | ##other) |
@@ -877,6 +1085,12 @@ class SchemaVisitor(object):
               {any attributes with non-schema Namespace...}>
             Content: (annotation?)
             </anyAttribute>
+
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         process_contents = node.get('processContents', 'strict')
         return xsd_elements.AnyAttribute(process_contents=process_contents)
@@ -886,6 +1100,8 @@ class SchemaVisitor(object):
         non-XML data within an XML document. An XML Schema notation declaration
         is a reconstruction of XML 1.0 NOTATION declarations.
 
+        Definition::
+
             <notation
               id = ID
               name = NCName
@@ -895,17 +1111,18 @@ class SchemaVisitor(object):
             Content: (annotation?)
             </notation>
 
+        :param node: The XML node
+        :type node: lxml.etree._Element
+        :param parent: The parent XML node
+        :type parent: lxml.etree._Element
+
         """
         pass
 
     def _get_type(self, name):
         assert name is not None
         name = self._create_qname(name)
-        try:
-            retval = self.schema.get_type(name)
-        except (exceptions.NamespaceError, exceptions.LookupError):
-            retval = xsd_types.UnresolvedType(name, self.schema)
-        return retval
+        return UnresolvedType(name, self.schema)
 
     def _create_qname(self, name):
         if not isinstance(name, etree.QName):
@@ -922,7 +1139,7 @@ class SchemaVisitor(object):
         # referenced.
         if (
             name.namespace == 'http://schemas.xmlsoap.org/soap/encoding/' and
-            name.namespace not in self.document._imports
+            not self.document.is_imported(name.namespace)
         ):
             import_node = etree.Element(
                 tags.import_,
@@ -943,11 +1160,14 @@ class SchemaVisitor(object):
     def _process_attributes(self, node, items):
         attributes = []
         for child in items:
-            attribute = self.process(child, node)
             if child.tag in (tags.attribute, tags.attributeGroup, tags.anyAttribute):
+                attribute = self.process(child, node)
                 attributes.append(attribute)
             else:
-                raise XMLParseError("Unexpected tag: %s" % child.tag)
+                raise XMLParseError(
+                    "Unexpected tag `%s`" % (child.tag),
+                    filename=self.document._location,
+                    sourceline=node.sourceline)
         return attributes
 
     visitors = {
